/*
 * $Log: AccountEntityBean.java,v $
 * Revision 1.2  2005/03/17 05:24:39  glz
 * First full FT version (no DB duplication handling yet)
 *
 * Revision 1.1  2005/03/15 01:17:25  ssimanta
 * Imported VaultProject to FTVaultProject
 *
 * Revision 1.2  2005/02/16 02:27:15  ssimanta
 * Made changes for Transaction-Account relationship.
 *
 * Revision 1.1  2005/02/13 21:13:00  ssimanta
 * Initial commit for Account Beans
 *
 */
package edu.cmu.mse.vault.account;

import java.rmi.RemoteException;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import edu.cmu.mse.vault.customer.CustomerEntityLocal;
import edu.cmu.mse.vault.message.Message;
import edu.cmu.mse.vault.message.MessageEntity;
import edu.cmu.mse.vault.message.MessageEntityHome;
import edu.cmu.mse.vault.message.MessageKey;

/**
 * $Version: $
 * 
 * @ejb.bean name="AccountEntity" display-name="Name for AccountEntity"
 *           description="Description for AccountEntity"
 *           jndi-name="ejb/AccountEntity" type="CMP" cmp-version="2.x"
 *           view-type="both"
 */
public abstract class AccountEntityBean implements EntityBean {

    public abstract void setBalance(Double balance);

    public abstract Double getBalance();
    
    /** Rememeber accountId is the auto-generated key  */
    public abstract void setAccountId(Integer accountId);

    public abstract Integer getAccountId();
    
    /** Relationship methods for account-customer relation */
    public abstract void setCustomer(CustomerEntityLocal customer);
    public abstract CustomerEntityLocal getCustomer();
    
    
    //Defined for collection-based relationship between Account and Transaction EJBs.
    //One Account EJB can have multiple Transaction EJBs. 
    //Also note that we are using a Collection becuase one customer
    //can have mutliple accounts in the account table.
    public abstract Collection getTransactions();

    public abstract void setTransactions(Collection transactions);    
    
    /**
     *  
     */
    public AccountEntityBean() {
        super();
        // TODO Auto-generated constructor stub
    }

    /**
     * This class returns Object becuase the Primary key for account table in
     * the vault database is an AUTO-INCREMENT field
     */
    public Object ejbCreate(Account account, CustomerEntityLocal customerEntity) throws CreateException,
            RemoteException {
        System.out.println("AccountEntityBean.ejbCreate()");

		//We just need to set the balance here. 
        this.setBalance(account.getBalance());

        return this.getAccountId();
    }

    public void ejbPostCreate(Account account, CustomerEntityLocal customerEntity) throws CreateException,
            RemoteException {
        
        System.out.println("AccountEntityBean.ejbPostCreate()");
        /*IMP: This is a relationship and hence it cannot be done in ejbCreate() it has to be done in 
         *     in the ejbPostCreate() method. 
         */
        this.setCustomer(customerEntity);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.ejb.EntityBean#ejbActivate()
     */
    public void ejbActivate() throws EJBException, RemoteException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.ejb.EntityBean#ejbLoad()
     */
    public void ejbLoad() throws EJBException, RemoteException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.ejb.EntityBean#ejbPassivate()
     */
    public void ejbPassivate() throws EJBException, RemoteException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.ejb.EntityBean#ejbRemove()
     */
    public void ejbRemove() throws RemoveException, EJBException,
            RemoteException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.ejb.EntityBean#ejbStore()
     */
    public void ejbStore() throws EJBException, RemoteException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.ejb.EntityBean#setEntityContext(javax.ejb.EntityContext)
     */
    public void setEntityContext(EntityContext ctx) throws EJBException,
            RemoteException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.ejb.EntityBean#unsetEntityContext()
     */
    public void unsetEntityContext() throws EJBException, RemoteException {
        // TODO Auto-generated method stub

    }

}