package edu.cmu.mse.vault.client;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Iterator;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;

import edu.cmu.mse.vault.account.Account;
import edu.cmu.mse.vault.account.AccountAgentSession;
import edu.cmu.mse.vault.account.AccountAgentSessionHome;
import edu.cmu.mse.vault.customer.Customer;
import edu.cmu.mse.vault.customer.CustomerAgentSession;
import edu.cmu.mse.vault.customer.CustomerAgentSessionHome;
import edu.cmu.mse.vault.customer.InvalidCustomerException;
import edu.cmu.mse.vault.message.Message;
import edu.cmu.mse.vault.message.MessageKey;
import edu.cmu.mse.vault.transaction.InsufficientAmountException;
import edu.cmu.mse.vault.transaction.InvalidAccountException;
import edu.cmu.mse.vault.transaction.InvalidAmountException;
import edu.cmu.mse.vault.transaction.TransactionAgentSession;
import edu.cmu.mse.vault.transaction.TransactionAgentSessionHome;
import edu.cmu.mse.vault.transaction.TransactionRow;
import edu.cmu.mse.vault.util.VaultUtil;

/**
 * @author Soumya
 *  
 */
public class VaultClient {

	public static void main(String[] args) {

		System.out.println("Vault client invoked ...");
		
		//FIX: Its a hack !!
		if( args.length == 1)
			; //do nothing as you are going to call history
		//Check for the valid number of arguments before doing anything.
		else if (args.length < 3 || args.length > 4) {
			System.out
					.println("Usage: ant run.addcustomer <addcustomer> <firstname> <lastname>");
			System.out
					.println("Usage: ant run.editcustomer <editcustomer> <customerid> <firstname> <lastname>");
			System.out
					.println("Usage: ant run.addaccount <editcustomer> <customerid> <firstname> <lastname>");
			System.out
					.println("Usage: ant run.deposit <deposit> <accountid> <amountOfMoney>");
			System.out
					.println("Usage: ant run.withdraw <withdraw> <accountid> <amountOfMoney>");
			System.out
					.println("Usage: ant run.transfer <transfer> <sourceaccountid> <destinationaccountid> <amountOfMoney>");

			System.out.println("Exiting ..... Good bye !!!");
			System.exit(1);
		}

		VaultClient client = new VaultClient();
		System.out.println("args[0]:" + args[0]);
		System.out.println("args[1]:" + args[1]);
		System.out.println("args[2]:" + args[2]);
		if (args.length > 3)
			System.out.println("args[3]:" + args[3]);

		boolean result = false;
		if (args[0].equals("addcustomer"))
			result = client.addCustomer(args[1], args[2]);
		else if (args[0].equals("editcustomer"))
			result = client.editCustomer(args[1], args[2], args[3]);
		else if (args[0].equals("addaccount"))
			result = client.addAccount(args[1], args[2]);
		else if (args[0].equals("deposit"))
			result = client.deposit(args[1], args[2], true);
		else if (args[0].equals("withdraw"))
			result = client.deposit(args[1], args[2], false);
		else if (args[0].equals("transfer"))
			result = client.transfer(args[1], args[2], args[3]);
		else if (args[0].equals("history"))
			result = client.showTransactionHistory(args[1]);

		if (result)
			System.out.println("The operation completed successfully ....");
		else
			System.out
					.println("The operation couldn't complete successfully ..... ");

	}

	public boolean showTransactionHistory(String accountId) {
		if (VaultUtil.isNullOrBlank(accountId)) {
			System.out.println("The account id cannot be blank ...");
			return false;
		}
		
		int iAccountId = -1;
		
		//get the number values from the strings passed
		try {

			iAccountId = Integer.parseInt(accountId);
			
		} catch (NumberFormatException nfe) {

			System.out.println("The account id passed " + accountId + " is not a number ");
			return false;

		}
		
		boolean status = false;

		try {
			Context jndiContext = getInitialContext();
			TransactionAgentSessionHome home = (TransactionAgentSessionHome)jndiContext.lookup("TransactionAgentSessionEJB");
			TransactionAgentSession accountSession = home.create();

			List transactionData = ((List)((Message)accountSession.getTransactionHistory(nextKey(), iAccountId)).getPayload());
			
			Iterator iter = transactionData.iterator();
			TransactionRow currentRow = null;
			while (iter.hasNext()) {
				currentRow = (TransactionRow) iter.next();
				System.out.println(currentRow.toString());
			}
			status = true;

		} catch (java.rmi.RemoteException re) {
			status = false;
			re.printStackTrace();
		} catch (javax.naming.NamingException ne) {
			status = false;
			ne.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			status = false;
			ce.printStackTrace();
		} 
		
		return status;

	}

	public boolean deposit(String accountid, String amountOfMoney,
			boolean deposit) {
		//      public boolean addAccount(String strCustomerId, String strBalance) {
		if (VaultUtil.isNullOrBlank(accountid)) {
			System.out.println("The account Id cannnot be blank...");
			return false;
		}

		if (VaultUtil.isNullOrBlank(amountOfMoney)) {
			System.out
					.println("The amount for this account cannot be empty...");
			return false;
		}

		int iAccountId = -1;
		double dAmountOfMoney = 0.0;

		//get the number values from the strings passed
		try {

			iAccountId = Integer.parseInt(accountid);
			dAmountOfMoney = Double.parseDouble(amountOfMoney);

		} catch (NumberFormatException nfe) {

			System.out.println("One of the inputs account id " + accountid
					+ " or the amount " + amountOfMoney
					+ " are not numbers ...");
			return false;

		}

		boolean status = false;

		try {
			Context jndiContext = getInitialContext();
			AccountAgentSessionHome home = (AccountAgentSessionHome)jndiContext.lookup("AccountAgentSessionEJB");
			AccountAgentSession accountSession = home.create();

			double newBalance;

			if (deposit)
				newBalance = ((Double)((Message)accountSession.depositMoney(nextKey(), iAccountId,
						dAmountOfMoney)).getPayload()).doubleValue();
			else
				newBalance = ((Double)((Message)accountSession.withdrawMoney(nextKey(), iAccountId,
						dAmountOfMoney)).getPayload()).doubleValue();

			System.out.println((deposit ? "Deposit" : "Withdrawal")
					+ " was made, the new balance is : "
					+ Double.toString(newBalance));

			status = true;

		} catch (java.rmi.RemoteException re) {
			status = false;
			re.printStackTrace();
		} catch (javax.naming.NamingException ne) {
			status = false;
			ne.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			status = false;
			ce.printStackTrace();
		} catch (InvalidAccountException iae) {
			status = false;
			iae.printStackTrace();
		} catch (InvalidAmountException iame) {
			status = false;
			iame.printStackTrace();
		} catch (InsufficientAmountException e) {
			status = false;
			e.printStackTrace();
		}

		return status;
	}

	public boolean addCustomer(String firstName, String lastName) {

		if (firstName == null) {
			System.out.println("First name cannot be blank...");
			return false;
		}

		if (lastName == null) {
			System.out.println("Last name cannot be blank...");
			return false;
		}
		boolean status = false;
		try {
			Context jndiContext = getInitialContext();
			CustomerAgentSessionHome home = (CustomerAgentSessionHome)jndiContext.lookup("CustomerAgentSessionEJB");
			CustomerAgentSession customerSession = home.create();

			Customer customer = new Customer(firstName, lastName);

			int custId = ((Integer)((Message)customerSession.createCustomer(nextKey(), customer)).getPayload()).intValue();

			System.out
					.println("Added the client with customer id -> " + custId);

			status = true;

		} catch (java.rmi.RemoteException re) {
			status = false;
			re.printStackTrace();
		} catch (javax.naming.NamingException ne) {
			status = false;
			ne.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			status = false;
			ce.printStackTrace();
		}

		return status;
	}

	public boolean editCustomer(String strCustomerId, String firstName,
			String lastName) {
		if (firstName == null) {
			System.out.println("First name cannot be blank...");
			return false;
		}

		if (lastName == null) {
			System.out.println("Last name cannot be blank...");
			return false;
		}

		int customerId = -1;

		try {
			customerId = Integer.parseInt(strCustomerId);
		} catch (NumberFormatException nfe) {
			//set the custId to -1 here so that you can show the error message
			// below.
			customerId = -1;
		}

		if (customerId < 0) {
			System.out.print("customer Id " + strCustomerId
					+ "to be edited is not a valid number ....");
			return false;
		}

		boolean status = false;
		try {
			Context jndiContext = getInitialContext();
			//IMP: CustomerAgentSessionEJB this much exactly MAP the JNDI name
			// in the
			//     src/META-INF/jboss.xml
			CustomerAgentSessionHome home = (CustomerAgentSessionHome)jndiContext.lookup("CustomerAgentSessionEJB");
			CustomerAgentSession customerSession = home.create();

			Customer customer = new Customer(firstName, lastName);
			status = ((Boolean)((Message)customerSession.editCustomer(nextKey(), customerId, customer)).getPayload()).booleanValue();

			
			if (status)
				System.out
						.println("Successfully edited the customer with customerid-> "
								+ customerId);
			else
				System.out
						.println("Couldn't edit the customer with customerId -> "
								+ customerId);

		} catch (java.rmi.RemoteException re) {
			status = false;
			re.printStackTrace();
		} catch (javax.naming.NamingException ne) {
			status = false;
			ne.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			status = false;
			ce.printStackTrace();
		} catch (InvalidCustomerException ivce) {
			status = false;
			ivce.printStackTrace();
		}

		return status;
	}

	public static Context getInitialContext()
			throws javax.naming.NamingException {
		return new InitialContext();

	}

	/**
	 * Is used to create a new account for an existing customer
	 * 
	 * @param strCustomerId
	 * @param strBalance
	 * @return
	 */
	public boolean addAccount(String strCustomerId, String strBalance) {

		if (VaultUtil.isNullOrBlank(strCustomerId)) {
			System.out.println("The Customer Id cannnot be blank...");
			return false;
		}

		if (VaultUtil.isNullOrBlank(strBalance)) {
			System.out
					.println("The balance for this account cannot be empty...");
			return false;
		}

		int iCustId = -1;
		double dBalance = 0.0;

		//get the number values from the strings passed
		try {

			iCustId = Integer.parseInt(strCustomerId);
			dBalance = Double.parseDouble(strBalance);

		} catch (NumberFormatException nfe) {

			System.out.println("One of the inputs customer id " + strCustomerId
					+ " or the balance " + strBalance + " are not numbers ...");
			return false;

		}

		boolean status = false;

		try {
			Context jndiContext = getInitialContext();
			AccountAgentSessionHome home = (AccountAgentSessionHome)jndiContext.lookup("AccountAgentSessionEJB");
			AccountAgentSession accountSession = home.create();

			Account account = new Account(new Integer(iCustId), new Double(
					dBalance));

			int accountId = ((Integer)((Message)accountSession.createAccount(nextKey(), iCustId, account)).getPayload()).intValue();

			System.out.println("New account with account id -> " + accountId
					+ " created for customer -> " + iCustId);

			status = true;

		} catch (java.rmi.RemoteException re) {
			status = false;
			re.printStackTrace();
		} catch (javax.naming.NamingException ne) {
			status = false;
			ne.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			status = false;
			ce.printStackTrace();
		} catch (InvalidCustomerException ice) {
			status = false;
			ice.printStackTrace();
		}

		return status;
	}

	public boolean transfer(String sourceaccountid, String destaccountid,
			String amountOfMoney) {
		//      public boolean addAccount(String strCustomerId, String strBalance) {
		if (VaultUtil.isNullOrBlank(sourceaccountid)
				|| VaultUtil.isNullOrBlank(destaccountid)) {
			System.out.println("The account Id cannnot be blank...");
			return false;
		}

		if (VaultUtil.isNullOrBlank(amountOfMoney)) {
			System.out
					.println("The amount for this account cannot be empty...");
			return false;
		}

		int iSourceAccountId = -1;
		int iDestAccountId = -1;
		double dAmountOfMoney = 0.0;

		//get the number values from the strings passed
		try {

			iSourceAccountId = Integer.parseInt(sourceaccountid);
			iDestAccountId = Integer.parseInt(destaccountid);
			dAmountOfMoney = Double.parseDouble(amountOfMoney);

		} catch (NumberFormatException nfe) {

			System.out.println("One of the inputs account id "
					+ sourceaccountid + " or destination account it"
					+ destaccountid + " or the amount " + amountOfMoney
					+ " are not numbers ...");
			return false;

		}

		boolean status = false;

		try {
			Context jndiContext = getInitialContext();
			AccountAgentSessionHome home = (AccountAgentSessionHome)jndiContext.lookup("AccountAgentSessionEJB");
			AccountAgentSession accountSession = home.create();

			double newBalance = ((Double)((Message)accountSession.transferMoney(nextKey(), iSourceAccountId,
					iDestAccountId, dAmountOfMoney)).getPayload()).doubleValue();

			System.out
					.println("TransferMoney was made, the new balance of destination account is : "
							+ Double.toString(newBalance));

			status = true;

		} catch (java.rmi.RemoteException re) {
			status = false;
			re.printStackTrace();
		} catch (javax.naming.NamingException ne) {
			status = false;
			ne.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			status = false;
			ce.printStackTrace();
		} catch (InvalidAccountException iae) {
			status = false;
			iae.printStackTrace();
		} catch (InvalidAmountException iame) {
			status = false;
			iame.printStackTrace();
		} catch (InsufficientAmountException e) {
			status = false;
			e.printStackTrace();
		}

		return status;
	}
    
    public boolean depositMoney(String accountID, String amount) {
        return deposit(accountID, amount, true);
    }

    public boolean withdrawMoney(String accountID, String amount) {
        return deposit(accountID, amount, false);
    }

    public boolean transferMoney(String fromAccountID, String toAccountID, String amount) {
        return transfer(fromAccountID, toAccountID, amount);
    }

	public final static String cfgFn = "VaultClient.cfg";

	public MessageKey nextKey() {
		MessageKey mk = null;
		try {
			FileReader fr = new FileReader(cfgFn);
			BufferedReader br = new BufferedReader(fr);
			String clientID = br.readLine();
			int seqNum = Integer.parseInt(br.readLine());
			br = null;
			fr.close();
			fr = null;
			
			mk = new MessageKey(clientID, seqNum);
			System.out.println("nextKeyCreated with value: (" + clientID + ", " + seqNum + ")");
			seqNum++;
			
			FileWriter fw = new FileWriter(cfgFn);
			BufferedWriter bw = new BufferedWriter(fw);
			bw.write(clientID);
			bw.newLine();
			bw.write(Integer.toString(seqNum));
			bw.newLine();
			bw.flush();
			bw = null;
			fw.close();
			fw = null;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		
		return mk;
	}
}