package edu.cmu.mse.vault.client;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import edu.cmu.mse.vault.account.Account;
import edu.cmu.mse.vault.account.AccountAgentSession;
import edu.cmu.mse.vault.account.AccountAgentSessionHome;
import edu.cmu.mse.vault.customer.Customer;
import edu.cmu.mse.vault.customer.CustomerAgentSession;
import edu.cmu.mse.vault.customer.CustomerAgentSessionHome;
import edu.cmu.mse.vault.customer.InvalidCustomerException;
import edu.cmu.mse.vault.message.Message;
import edu.cmu.mse.vault.message.MessageKey;
import edu.cmu.mse.vault.transaction.InsufficientAmountException;
import edu.cmu.mse.vault.transaction.InvalidAccountException;
import edu.cmu.mse.vault.transaction.InvalidAmountException;
import edu.cmu.mse.vault.transaction.TransactionAgentSession;
import edu.cmu.mse.vault.transaction.TransactionAgentSessionHome;
import edu.cmu.mse.vault.transaction.TransactionRow;
import edu.cmu.mse.vault.util.VaultUtil;

/**
 * @author Jinhee Lee
 *  
 */
/**
 * @author Jinhee Lee
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class VaultFTBaseClient {

	/**
	 * Comment for <code>vjc</code> VaultJNDICacheManager is the static member
	 * of VaultFTBaseClient, because VaultFTBaseClient need to access to the
	 * CacheMap to retrieve the object reference and JNDI name
	 */
	public static VaultJNDICacheManager vjc;
	
	/**
	 * Comment for <code>refID</code>
	 * unique reference ID, which are forwarded from the VauleWorkThread
	 * We can get the JNDI information with this integer value
	 */
	private int refID = 0;
	private String strRefID = null;
	private boolean successfulConnection = true;
	
	/**
	 * @param refID
	 */
	public VaultFTBaseClient(int refID) {
		super();
		this.refID = refID;
		this.strRefID = Integer.toString(refID);
	}
	
	
	
	/**
	 * @return Returns the successfulConnection.
	 */
	public boolean isSuccessfulConnection() {
		return successfulConnection;
	}
	/**
	 * @param accountId
	 * @return true if the transaction is done successfully
	 */
	public boolean showTransactionHistory(String accountId) {
		if (VaultUtil.isNullOrBlank(accountId)) {
			System.out.println("<ClientThread" + refID +">: The account id cannot be blank ...");
			return false;
		}
		
		int iAccountId = -1;
		
		//get the number values from the strings passed
		try {

			iAccountId = Integer.parseInt(accountId);
			
		} catch (NumberFormatException nfe) {

			System.out.println("<ClientThread" + refID +">: The accountid " + accountId + " is not a number ");
			return false;

		}
		
		boolean status = false;

		try {
			
			/*
			 * retrieve the object reference and jndi full name with reference
			 * ID from the JNDICacheManager
			 */
			
			ArrayList ref = (ArrayList)VaultFTBaseClient.vjc.getTranObjectReference(strRefID);
			if (ref == null){
				System.out.println("<ClientThread" + refID +">: showTransactionHistory: no JNDI name in the Cache");
				return false;
			}
			System.out.println("<ClientThread" + refID +">: showTransactionHistory: JNDIName=" + ref.get(0));
			TransactionAgentSessionHome home = (TransactionAgentSessionHome) PortableRemoteObject
					.narrow(ref.get(1), TransactionAgentSessionHome.class);
			if (home == null){
				System.out.println("<ClientThread" + refID +">: showTransactionHistory: TransactionAgentSessionHome is null");
				return false;
			}
			TransactionAgentSession accountSession = home.create();
			if (accountSession == null){
				System.out.println("<ClientThread" + refID +">: showTransactionHistory: TransactionAgentSession is null");
				return false;
			}
			//List transactionData = accountSession.getTransactionHistory( iAccountId );
			List transactionData = ((List)((Message)accountSession.getTransactionHistory(nextKey(), iAccountId)).getPayload());
			
			Iterator iter = transactionData.iterator();
			TransactionRow currentRow = null;
			while (iter.hasNext()) {
				currentRow = (TransactionRow) iter.next();
				System.out.println(currentRow.toString());
			}
			status = true;

		} catch (java.rmi.RemoteException re) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: showTransactionHistory: RemoteException occurred");
			re.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: showTransactionHistory: CreateException occurred");
			ce.printStackTrace();
		} 
		
		return status;

	}

	/**
	 * @param accountid
	 * @param amountOfMoney
	 * @param deposit
	 * @return true if the transaction is done successfully
	 */
	public boolean deposit(String accountid, String amountOfMoney,
			boolean deposit) {
		
		// check the sanity of the argument
		if (VaultUtil.isNullOrBlank(accountid)) {
			System.out.println("<ClientThread" + refID +">: deposit: The account Id cannnot be blank...");
			return false;
		}

		if (VaultUtil.isNullOrBlank(amountOfMoney)) {
			System.out
					.println("<ClientThread" + refID +">: deposit: The amount for this account cannot be empty...");
			return false;
		}

		int iAccountId = -1;
		double dAmountOfMoney = 0.0;

		//get the number values from the strings passed
		try {

			iAccountId = Integer.parseInt(accountid);
			dAmountOfMoney = Double.parseDouble(amountOfMoney);

		} catch (NumberFormatException nfe) {

			System.out.println("<ClientThread" + refID +">: deposit: One of the inputs account id " + accountid
					+ " or the amount " + amountOfMoney
					+ " are not numbers ...");
			return false;

		}

		boolean status = false;

		try {
			
			/*
			 * retrieve the object reference and jndi full name with reference
			 * ID from the JNDICacheManager
			 */
			ArrayList ref = (ArrayList)VaultFTBaseClient.vjc.getAccObjectReference(strRefID);
			if (ref == null){
				System.out.println("<ClientThread" + refID +">: deposit: no JNDI name in the Cache");
				return false;
			}
			System.out.println("<ClientThread" + refID +">: deposit: JNDIName=" + ref.get(0));
			AccountAgentSessionHome home = (AccountAgentSessionHome) PortableRemoteObject
					.narrow(ref.get(1), AccountAgentSessionHome.class);
			if (home == null){
				System.out.println("<ClientThread" + refID +">: deposit: AccountAgentSessionHome is null");
				return false;
			}
			AccountAgentSession accountSession = home.create();
			if (accountSession == null){
				System.out.println("<ClientThread" + refID +">: deposit: AccountAgentSession is null");
				return false;
			}
			double newBalance;
			if (deposit)
				newBalance = ((Double)((Message)accountSession.depositMoney(new MessageKey("Client", 1), iAccountId,
						dAmountOfMoney)).getPayload()).doubleValue();
			else
				newBalance = ((Double)((Message)accountSession.withdrawMoney(new MessageKey("Client", 1), iAccountId,
						dAmountOfMoney)).getPayload()).doubleValue();

			System.out.println("<ClientThread" + refID +">: " + (deposit ? "Deposit" : "Withdrawal")
					+ " was made, the new balance is : "
					+ Double.toString(newBalance));

			status = true;

		} catch (java.rmi.RemoteException re) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: deposit: RemoteException occurred");
			re.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: deposit: CreateException occurred");
			ce.printStackTrace();
		} catch (InvalidAccountException iae) {
			successfulConnection = true;
			status = false;
			System.out.println("<ClientThread" + refID +">: deposit: InvalidAccountException occurred");
			iae.printStackTrace();
		} catch (InvalidAmountException iame) {
			successfulConnection = true;
			status = false;
			System.out.println("<ClientThread" + refID +">: deposit: InvalidAmountException occurred");
			iame.printStackTrace();
		} catch (InsufficientAmountException e) {
			successfulConnection = true;
			status = false;
			System.out.println("<ClientThread" + refID +">: deposit: InsufficientAmountException occurred");
			e.printStackTrace();
		}

		return status;
	}

	/**
	 * create new customer information to the customer database
	 * @param firstName
	 * @param lastName
	 * @return true if the transaction is done successfully, false if not
	 */
	public boolean addCustomer(String firstName, String lastName) {

		if (firstName == null) {
			System.out.println("<ClientThread" + refID +">: First name cannot be blank...");
			return false;
		}

		if (lastName == null) {
			System.out.println("<ClientThread" + refID +">: Last name cannot be blank...");
			return false;
		}
		
		System.out.println("<ClientThread" + refID +">: Start adding customer Firstname:" + firstName + " Lastname:" +lastName );
		boolean status = false;
		try {
			
			/*
			 * retrieve the object reference and jndi full name with reference
			 * ID from the JNDICacheManager
			 */
			ArrayList ref = (ArrayList)VaultFTBaseClient.vjc.getCustObjectReference(strRefID);
			if (ref == null){
				System.out.println("<ClientThread" + refID +">: addCustomer: no JNDI name in the Cache");
				return false;
			}
			System.out.println("<ClientThread" + refID +">: addCustomer: JNDIName=" + ref.get(0));
			CustomerAgentSessionHome home = (CustomerAgentSessionHome) PortableRemoteObject
					.narrow(ref.get(1), CustomerAgentSessionHome.class);
			
			if (home == null){
				System.out.println("<ClientThread" + refID +">: addCustomer: CustomerAgentSessionHome is null");
				return false;
			}
			
			CustomerAgentSession customerSession = home.create();

			if (customerSession == null){
				System.out.println("<ClientThread" + refID +">: addCustomer: CustomerAgentSession is null");
				return false;
			}
			
			Customer customer = new Customer(firstName, lastName);
			int custId = ((Integer)((Message)customerSession.createCustomer(new MessageKey("Client", 1), customer)).getPayload()).intValue();

			System.out.println("<ClientThread" + refID +">: addCustomer: Added the client with customer id -> " + custId);

			status = true;

		} catch (java.rmi.RemoteException re) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: addCustomer: RemoteException occurred");
			re.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: addCustomer: CreateException occurred");
			ce.printStackTrace();
		}

		return status;
	}

	public boolean editCustomer(String strCustomerId, String firstName,
			String lastName) {
		if (firstName == null) {
			System.out.println("<ClientThread" + refID +">: editCustomer: First name cannot be blank...");
			return false;
		}

		if (lastName == null) {
			System.out.println("<ClientThread" + refID +">: editCustomer: Last name cannot be blank...");
			return false;
		}

		int customerId = -1;

		try {
			customerId = Integer.parseInt(strCustomerId);
		} catch (NumberFormatException nfe) {
			//set the custId to -1 here so that you can show the error message
			// below.
			customerId = -1;
		}

		if (customerId < 0) {
			System.out.print("<ClientThread" + refID +">: editCustomer: customer Id " + strCustomerId
					+ "to be edited is not a valid number ....");
			return false;
		}

		boolean status = false;
		try {
			
			/*
			 * retrieve the object reference and jndi full name with reference
			 * ID from the JNDICacheManager
			 */
			ArrayList ref = (ArrayList)VaultFTBaseClient.vjc.getCustObjectReference(strRefID);
			if (ref == null){
				System.out.println("<ClientThread" + refID +">: editCustomer: no JNDI name in the Cache");
				return false;
			}
			
			System.out.println("<ClientThread" + refID +">: editCustomer: JNDIName=" + ref.get(0));
			CustomerAgentSessionHome home = (CustomerAgentSessionHome) PortableRemoteObject
					.narrow(ref.get(1), CustomerAgentSessionHome.class);
			if (home == null){
				System.out.println("<ClientThread" + refID +">: editCustomer: CustomerAgentSessionHome is null");
				return false;
			}
			CustomerAgentSession customerSession = home.create();
			if (customerSession == null){
				System.out.println("<ClientThread" + refID +">: editCustomer: CustomerAgentSession is null");
				return false;
			}
			//Customer customer = new Customer(firstName, lastName);
			//status = customerSession.editCustomer(customerId, customer);
			Customer customer = new Customer(firstName, lastName);
			status = ((Boolean)((Message)customerSession.editCustomer(nextKey(), customerId, customer)).getPayload()).booleanValue();

			
			
			if (status)
				System.out.println("<ClientThread" + refID +">: editCustomer: Successfully edited the customer with customerid-> "
								+ customerId);
			else
				System.out.println("<ClientThread" + refID +">: editCustomer: Couldn't edit the customer with customerId -> "
								+ customerId);

		} catch (java.rmi.RemoteException re) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: editCustomer: RemoteException occurred");
			re.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: editCustomer: CreateException occurred");
			ce.printStackTrace();
		} catch (InvalidCustomerException ivce) {
			successfulConnection = true;
			status = false;
			System.out.println("<ClientThread" + refID +">: editCustomer: InvalidCustomerException occurred");
			ivce.printStackTrace();
		}

		return status;
	}

	/**
	 * @return
	 * @throws javax.naming.NamingException
	 */
	public static Context getInitialContext()
			throws javax.naming.NamingException {
		return new InitialContext();

	}

	/**
	 * Is used to create a new account for an existing customer
	 * 
	 * @param strCustomerId
	 * @param strBalance
	 * @return
	 */
	public boolean addAccount(String strCustomerId, String strBalance) {

		if (VaultUtil.isNullOrBlank(strCustomerId)) {
			System.out.println("<ClientThread" + refID +">: addAccount: The Customer Id cannnot be blank...");
			return false;
		}

		if (VaultUtil.isNullOrBlank(strBalance)) {
			System.out
					.println("<ClientThread" + refID +">: addAccount: The balance for this account cannot be empty...");
			return false;
		}

		int iCustId = -1;
		double dBalance = 0.0;

		//get the number values from the strings passed
		try {

			iCustId = Integer.parseInt(strCustomerId);
			dBalance = Double.parseDouble(strBalance);

		} catch (NumberFormatException nfe) {

			System.out.println("<ClientThread" + refID +">: addAccount: One of the inputs customer id " + strCustomerId
					+ " or the balance " + strBalance + " are not numbers ...");
			return false;

		}

		boolean status = false;

		try {
			
			/*
			 * retrieve the object reference and jndi full name with reference
			 * ID from the JNDICacheManager
			 */
			
			ArrayList ref = (ArrayList)VaultFTBaseClient.vjc.getAccObjectReference(strRefID);
			if (ref == null){
				System.out.println("<ClientThread" + refID +">: addAccount: no JNDI name in the Cache");
				return false;
			}
			System.out.println("<ClientThread" + refID +">: addAccount: JNDIName=" + ref.get(0));
			AccountAgentSessionHome home = (AccountAgentSessionHome) PortableRemoteObject
					.narrow(ref.get(1), AccountAgentSessionHome.class);

			if (home == null){
				System.out.println("<ClientThread" + refID +">: addAccount: AccountAgentSessionHome is null");
				return false;
			}
			AccountAgentSession accountSession = home.create();
			if (accountSession == null){
				System.out.println("<ClientThread" + refID +">: addAccount: AccountAgentSession is null");
				return false;
			}
			
			Account account = new Account(new Integer(iCustId), new Double(
					dBalance));

			int accountId = ((Integer)((Message)accountSession.createAccount(new MessageKey("Client", 1), iCustId, account)).getPayload()).intValue();

			System.out.println("<ClientThread" + refID +">: addAccount: New account with account id -> " + accountId
					+ " created for customer -> " + iCustId);

			status = true;

		} catch (java.rmi.RemoteException re) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: addAccount: RemoteException occurred");
			re.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: addAccount: CreateException occurred");
			ce.printStackTrace();
		} catch (InvalidCustomerException ice) {
			successfulConnection = true;			
			status = false;
			System.out.println("<ClientThread" + refID +">: addAccount: InvalidCustomerException occurred");
			ice.printStackTrace();
		}

		return status;
	}

	/**
	 * @param sourceaccountid
	 * @param destaccountid
	 * @param amountOfMoney
	 * @return
	 */
	public boolean transfer(String sourceaccountid, String destaccountid,
			String amountOfMoney) {
		//      public boolean addAccount(String strCustomerId, String strBalance) {
		if (VaultUtil.isNullOrBlank(sourceaccountid)
				|| VaultUtil.isNullOrBlank(destaccountid)) {
			System.out.println("<ClientThread" + refID +">: transfer: The account Id cannnot be blank...");
			return false;
		}

		if (VaultUtil.isNullOrBlank(amountOfMoney)) {
			System.out
					.println("<ClientThread" + refID +">: transfer: The amount for this account cannot be empty...");
			return false;
		}

		int iSourceAccountId = -1;
		int iDestAccountId = -1;
		double dAmountOfMoney = 0.0;

		//get the number values from the strings passed
		try {

			iSourceAccountId = Integer.parseInt(sourceaccountid);
			iDestAccountId = Integer.parseInt(destaccountid);
			dAmountOfMoney = Double.parseDouble(amountOfMoney);

		} catch (NumberFormatException nfe) {

			System.out.println("<ClientThread" + refID +">: transfer: One of the inputs account id "
					+ sourceaccountid + " or destination account it"
					+ destaccountid + " or the amount " + amountOfMoney
					+ " are not numbers ...");
			return false;

		}

		boolean status = false;

		try {
			/*
			 * retrieve the object reference and jndi full name with reference
			 * ID from the JNDICacheManager
			 */
			
			ArrayList ref = (ArrayList)VaultFTBaseClient.vjc.getAccObjectReference(strRefID);
			if (ref == null){
				System.out.println("<ClientThread" + refID +">: transfer: no JNDI name in the Cache");
				return false;
			}
			System.out.println("<ClientThread" + refID +">: transfer: JNDIName=" + ref.get(0));
			AccountAgentSessionHome home = (AccountAgentSessionHome) PortableRemoteObject
					.narrow(ref.get(1), AccountAgentSessionHome.class);
			if (home == null){
				System.out.println("<ClientThread" + refID +">: transfer: AccountAgentSessionHome is null");
				return false;
			}
			AccountAgentSession accountSession = home.create();
			if (accountSession == null){
				System.out.println("<ClientThread" + refID +">: transfer: AccountAgentSession is null");
				return false;
			}
			double newBalance = ((Double)((Message)accountSession.transferMoney(new MessageKey("Client", 1), iSourceAccountId,
					iDestAccountId, dAmountOfMoney)).getPayload()).doubleValue();

			System.out
					.println("<ClientThread" + refID +">: transfer: TransferMoney was made, the new balance of destination account is : "
							+ Double.toString(newBalance));

			status = true;

		} catch (java.rmi.RemoteException re) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: transfer: RemoteException occurred");
			re.printStackTrace();
		} catch (javax.ejb.CreateException ce) {
			successfulConnection = false;
			status = false;
			System.out.println("<ClientThread" + refID +">: transfer: CreateException occurred");
			ce.printStackTrace();
		} catch (InvalidAccountException iae) {
			successfulConnection = true;
			status = false;
			System.out.println("<ClientThread" + refID +">: transfer: InvalidAccountException occurred");
			iae.printStackTrace();
		} catch (InvalidAmountException iame) {
			successfulConnection = true;
			status = false;
			System.out.println("<ClientThread" + refID +">: transfer: InvalidAmountException occurred");
			iame.printStackTrace();
		} catch (InsufficientAmountException e) {
			successfulConnection = true;
			status = false;
			System.out.println("<ClientThread" + refID +">: transfer: InsufficientAmountException occurred");
			e.printStackTrace();
		}

		return status;
	}
    
    public boolean depositMoney(String accountID, String amount) {
        return deposit(accountID, amount, true);
    }

    public boolean withdrawMoney(String accountID, String amount) {
        return deposit(accountID, amount, false);
    }

    public boolean transferMoney(String fromAccountID, String toAccountID, String amount) {
        return transfer(fromAccountID, toAccountID, amount);
    }
    
	public final static String cfgFn = "VaultClient.cfg";

    public MessageKey nextKey() {
		MessageKey mk = null;
		try {
			FileReader fr = new FileReader(cfgFn);
			BufferedReader br = new BufferedReader(fr);
			String clientID = br.readLine();
			int seqNum = Integer.parseInt(br.readLine());
			br = null;
			fr.close();
			fr = null;
			
			mk = new MessageKey(clientID, seqNum);
			System.out.println("nextKeyCreated with value: (" + clientID + ", " + seqNum + ")");
			seqNum++;
			
			FileWriter fw = new FileWriter(cfgFn);
			BufferedWriter bw = new BufferedWriter(fw);
			bw.write(clientID);
			bw.newLine();
			bw.write(Integer.toString(seqNum));
			bw.newLine();
			bw.flush();
			bw = null;
			fw.close();
			fw = null;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		
		return mk;
	}
}