/*
 * Created on Mar 13, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package edu.cmu.mse.vault.client;

import java.io.*;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;

import edu.cmu.mse.vault.repman.IVaultReplicationManager;

/**
 * @author Jinhee Lee
 * 
 * VaultJNDICacheManager should retrieve the JNDI names and object references
 * from the JNDI server and Replica Manager peoriodically This class is a timer
 * thread class, whose run() method will be invoked with a fixed rate time.
 */
public class VaultJNDICacheManager implements Runnable {

	private Thread thread;

	// properties of JNDI interaction
	private int NUM_SERVER_INSTANCES = 3; //default value is 3

	private int PERIOD_CHECK_JNDI = 10000; //default value - 10 sec

	private String REPMANAGER_HOST;

	private int REPMANAGER_PORT;

	private VaultJNDICache JNDICache;

	private String JNDI_HOST;

	private List CustomerJNDIName;

	private List AccountJNDIName;

	private List TransactionJNDIName;

	private final String Vault_Rep_man = "Vault_Rep_man";

	private final String FileName = "vaultjndi.properties";

	public VaultJNDICacheManager() {

		thread = new Thread(this);
		thread.start();
	}

	/**
	 * @author Jinhee Lee
	 * 
	 * Inner class : VaultJNDICache This class contais three Hashtables for
	 * maintaining the object reference of all three sessionEJBs
	 */
	public class VaultJNDICache {

		/**
		 * Comment for <code>transactionCache</code> Hashtable for
		 * transactionSessionBean objects
		 */
		private Map transactionCache;

		/**
		 * Comment for <code>accountCache</code> Hashtable for
		 * accountSessionBean objects
		 */
		private Map accountCache;

		/**
		 * Comment for <code>customerCache</code> Hashtable for
		 * customerSessionBean objects
		 */
		private Map customerCache;

		/**
		 * VaultJNDICache constructor
		 */
		public VaultJNDICache() {
			/*
			 * Initialize all three hashtables
			 */
			this.transactionCache = new Hashtable();
			this.accountCache = new Hashtable();
			this.customerCache = new Hashtable();
		}

		/**
		 * addTransactionRef method is a method to add a new set of index(refID)
		 * as a key, the jndi string name and its object reference for the
		 * transaction session EJB
		 * 
		 * @param index :
		 *            key of the hashtable
		 * @param jndiName :
		 *            1st value of the hashtable
		 * @param transRef :
		 *            2nd value of the hashtable
		 */
		public void addTransactionRef(String index, Object jndiName,
				Object transRef) {
			/*
			 * Hashtable will have a string as a key, and ArrayList as a value
			 * ArrayList has two elements, the first one is Jndi name and the
			 * second one is transRef
			 */
			ArrayList obj = new ArrayList();
			obj.add(jndiName);
			obj.add(transRef);
			transactionCache.put(index, obj);
			System.out.println("<CACHE>: Transaction index="+ index+" jndiname="+jndiName);
		}

		/**
		 * addAccountRef method is a method to add a new set of index(refID) as
		 * a key, the jndi string name and its object reference for the account
		 * session EJB
		 * 
		 * @param index :
		 *            key of the hashtable
		 * @param jndiName :
		 *            1st value of the hashtable
		 * @param accRef :
		 *            2nd value of the hashtable
		 */
		public void addAccountRef(String index, Object jndiName, Object accRef) {
			ArrayList obj = new ArrayList();
			obj.add(jndiName);
			obj.add(accRef);
			accountCache.put(index, obj);
			System.out.println("<CACHE>: Account index="+ index+" jndiname="+jndiName);
		}

		/**
		 * addCustomerRef method is a method to add a new set of index(refID) as
		 * a key, the jndi string name and its object reference for the customer
		 * session EJB
		 * 
		 * @param index :
		 *            key of the hashtable
		 * @param jndiName :
		 *            1st value of the hashtable
		 * @param custRef :
		 *            2nd value of the hashtable
		 */
		public void addCustomerRef(String index, Object jndiName, Object custRef) {
			ArrayList obj = new ArrayList();
			obj.add(jndiName);
			obj.add(custRef);
			customerCache.put(index, obj);
			System.out.println("<CACHE>: Customer index="+ index+" jndiname="+jndiName);
		}

		/**
		 * clearCustomerCache This method remove all elements from the
		 * customerCache Hashtable
		 */
		public void clearCustomerCache() {
			customerCache.clear();
		}

		/**
		 * clearTransactionCache This method remove all elements from the
		 * transactionCache Hashtable
		 */
		public void clearTransactionCache() {
			transactionCache.clear();
		}

		/**
		 * accountCache This method remove all elements from the accountCache
		 * Hashtable
		 */
		public void clearAccountCache() {
			accountCache.clear();
		}

	}

	/**
	 * @param index :
	 *            reference ID of the work thread, Type: String
	 * @return ArrayList that contains the JNDI name string and SessionEJB
	 *         Object reference
	 */
	public Object getTranObjectReference(Object index) {
		return JNDICache.transactionCache.get(index);

	}

	/**
	 * @param index :
	 *            reference ID of the work thread Type: String
	 * @return ArrayList that contains the JNDI name string and SessionEJB
	 *         Object reference
	 */
	public Object getAccObjectReference(Object index) {
		return JNDICache.accountCache.get(index);

	}

	/**
	 * @param index :
	 *            reference ID of the work thread Type: String
	 * @return ArrayList that contains the JNDI name string and SessionEJB
	 *         Object reference
	 */
	public Object getCustObjectReference(Object index) {
		return JNDICache.customerCache.get(index);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {

		// Initialize JNDI Cache hashtable
		JNDICache = new VaultJNDICache();
		// Load properties from the property file
		System.out.println("<JNDI>: VaultJNDICacheManager Thread starts running");

		/*
		 * load all properties from properties file.
		 */
		loadProperties();

		
				
		while (true) {
			try {
				/*
				 * Lookup JNDI name and add the name and object reference to
				 * hashtables.
				 */
				if (lookupJndiContext() == false){
					System.out.println("<JNDI>: failed to lookup JNDI");
					break;
				}
				

				/*
				 * Set the timer and create timerthread to access the JNDI
				 * server periodically
				 */
				Thread.sleep(PERIOD_CHECK_JNDI);
				
			} catch (InterruptedException e) {
				System.out.println("<JNDI>: CacheManager loop is interrupted");
				e.printStackTrace();
				break;
			}
		}
	}

	/**
	 * Load properties from the vaultjndi.properties file. Now, the properties
	 * file contains three properties - MAX_NUM_SERVER_INSTANCES -
	 * PERIOD_CHECK_JNDI - PREFIX_JNDI_NAME
	 */
	public void loadProperties() {
		// Read properties file.
		Properties properties = new Properties();
		try {
			properties.load(new FileInputStream(FileName));

			/*
			 * NUM_SERVER_INSTANCES = Integer .valueOf( (String) properties
			 * .getProperty("NUM_SERVER_INSTANCES")) .intValue();
			 */

			REPMANAGER_HOST = properties.getProperty("REPMANAGER_HOST");
			if (REPMANAGER_HOST != null){
				System.out.println("<Properties>: REPMANAGER_HOST = "+ REPMANAGER_HOST);
			}
			else {
				System.out.println("<Properties>: REPMANAGER_HOST is not in the property file");
			}
				
			String strTemp = (String) properties.getProperty("REPMANAGER_PORT");
			if (strTemp != null) {
				REPMANAGER_PORT = Integer.valueOf(strTemp).intValue();
				System.out.println("<Properties>: REPMANAGER_PORT = "+ REPMANAGER_PORT);
			} else {
				System.out.println("<Properties>: REPMANAGER_PORT is not in the property file");
			}

			strTemp = (String) properties.getProperty("PERIOD_CHECK_JNDI");
			if (strTemp != null) {
				PERIOD_CHECK_JNDI = Integer.valueOf(strTemp).intValue();
				System.out.println("<Properties>: PERIOD_CHECK_JNDI = "+ PERIOD_CHECK_JNDI);
			} else {
				System.out.println("<Properties>: PERIOD_CHECK_JNDI is not in the property file");
			}

		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * This method connect to the replication manager and get JNDIName
	 */
	public boolean lookupRepManager() {
		boolean status = true;
		try {
			String rep_url = "rmi://" + REPMANAGER_HOST + ":" + REPMANAGER_PORT
			+ "/" + Vault_Rep_man;
			System.out.println("<REPMAN>: Try to access Replica manager Host: " + rep_url);
			
			IVaultReplicationManager rep = (IVaultReplicationManager) Naming.lookup(rep_url);
			
			if ( rep == null ){
				System.out.println("<REPMAN>: IVaultReplicationManager is null" );
				return false;
			}
			
			NUM_SERVER_INSTANCES = rep.getCurrentActiveReplicas();
			if ( NUM_SERVER_INSTANCES <= 0){
				System.out.println("<REPMAN>: There are no currently active replicas " );
				return false;
			}
			System.out.println("<REPMAN>: The no. of currently active replicas is " + NUM_SERVER_INSTANCES);
			JNDI_HOST = rep.getJNDIHostName();
			if ( JNDI_HOST == null){
				System.out.println("<REPMAN>: Replica manager returned null for the JNDIHostName");
				return false;
			}
			else {
				System.out.println("<REPMAN>: Replica manager returned " +  JNDI_HOST + " for the JNDIHostName");
			}
			CustomerJNDIName = rep.getCustomerRefNames();
			System.out.println("<REPMAN>: CustomerJNDIName: size " +  CustomerJNDIName.size());
			AccountJNDIName = rep.getAccountRefNames();
			System.out.println("<REPMAN>: AccountJNDIName: size " +  AccountJNDIName.size());
			TransactionJNDIName = rep.getTransactionRefNames();
			System.out.println("<REPMAN>: TransactionJNDIName: size " +  TransactionJNDIName.size());
			
			System.out.println("<REPMAN>: successfully done to access to replica manager");

		} catch (MalformedURLException e) {
			status = false;
			System.out.println("<REPMAN>: failed to access to replica manager: MalformedURLException");
			//e.printStackTrace();
			
		} catch (RemoteException e) {
			status = false;
			System.out.println("<REPMAN>: failed to access to replica manager: RemoteException");
			//e.printStackTrace();
		} catch (NotBoundException e) {
			status = false;
			System.out.println("<REPMAN>: failed to access to replica manager: NotBoundException");
			//e.printStackTrace();
		}

		return status;
	}

	/**
	 * Looking up JndiContext and object reference from the JNDI server All
	 * object reference will be added to the HashTable. We maintain three
	 * seperate HashTable for each EJBSession object.
	 */

	public boolean lookupJndiContext() {
		try {
			
			/*
			 * fetch the number of active server instances and a list of JNDI
			 * name of them
			 */
			if ( lookupRepManager() == false ){
				return false;
			}
			
			System.out
					.println("<JNDI>: Client is looking up JNDI object reference..");
			
			Context jndiContext = getInitialContext(JNDI_HOST);
			/*
			 * go through each list and look up the object ref by passing jndi
			 * name to jndi server. Index is the index of each cache table
			 */
			if (AccountJNDIName == null || AccountJNDIName.isEmpty()) {
				System.out
						.println("<JNDI>: AccountJNDIName: no JNDI name in the list");

			} else {
				JNDICache.clearAccountCache();
				
				Iterator iter = AccountJNDIName.iterator();
				int index = 0;
				while (iter.hasNext()) {
					index++;
					String jndiname = (String) iter.next();
					System.out.println("<JNDI>: AccountJNDIName: jndiname = " + jndiname);
					if ( jndiname == null ){
						continue;
					}
					
					Object accRef;
					try {
						accRef = jndiContext.lookup(jndiname);
						if ( accRef == null ){
							System.out.println("<JNDI>: AccountJNDIName: jndiContext is null, jndiname = " + jndiname);
							continue;
						}
				
					} catch (javax.naming.CommunicationException ce) {
						System.out.println("<JNDI>: AccountJNDIName: CommunicationException: failed to connect to "  + jndiname);
						continue;
					} catch (javax.naming.NameNotFoundException ce) {
						System.out.println("<JNDI>: AccountJNDIName: NameNotFoundException: failed to connect to "  + jndiname);
						continue;
					} 					
					
					JNDICache.addAccountRef(Integer.toString(index),
							(Object) jndiname, accRef);
				}
			}
			if (CustomerJNDIName == null || CustomerJNDIName.isEmpty()) {
				System.out
						.println("<JNDI>: CustomerJNDIName: no JNDI name in the list");

			} else {
				JNDICache.clearCustomerCache();
				
				Iterator iter = CustomerJNDIName.iterator();
				int index = 0;
				while (iter.hasNext()) {
					index++;
					String jndiname = (String) iter.next();
					System.out.println("<JNDI>: CustomerJNDIName: jndiname = " + jndiname);
					if ( jndiname == null ){
						continue;
					}
					
					Object custRef;
					try {
						custRef = jndiContext.lookup(jndiname);
						if ( custRef == null ){
							System.out.println("<JNDI>: CustomerJNDIName: jndiContext is null, jndiname = " + jndiname);
							continue;
						}
							
					} catch (javax.naming.CommunicationException ce) {
						System.out.println("<JNDI>: CustomerJNDIName: CommunicationException: failed to connect to "  + jndiname);
						continue;
					} catch (javax.naming.NameNotFoundException ce) {
						System.out.println("<JNDI>: CustomerJNDIName: NameNotFoundException: failed to connect to "  + jndiname);
						continue;
					} 		
					JNDICache.addCustomerRef(Integer.toString(index),
							(Object) jndiname, custRef);
				}
			}

			if (TransactionJNDIName == null || TransactionJNDIName.isEmpty()) {
				System.out
						.println("<JNDI>: TransactionJNDIName: no JNDI name in the list");

			} else {
				JNDICache.clearTransactionCache();
				
				int index = 0;
				Iterator iter = TransactionJNDIName.iterator();
				while (iter.hasNext()) {
					index++;
					String jndiname = (String) iter.next();
					System.out.println("<JNDI>: TransactionJNDIName: jndiname = " + jndiname);
					if ( jndiname == null ){
						continue;
					}
					
					Object trxRef;
					try {
						trxRef = jndiContext.lookup(jndiname);
						if ( trxRef == null ){
							System.out.println("<JNDI>: TransactionJNDIName: jndiContext is null, jndiname = " + jndiname);
							continue;
						}
					} catch (javax.naming.CommunicationException ce) {
						System.out.println("<JNDI>: TransactionJNDIName: CommunicationException: failed to connect to "  + jndiname);
						continue;
					} catch (javax.naming.NameNotFoundException ce) {
						System.out.println("<JNDI>: TransactionJNDIName: NameNotFoundException: failed to connect to "  + jndiname);
						continue;
					} 		
					
					JNDICache.addTransactionRef(Integer.toString(index),
							(Object) jndiname, trxRef);
				}
			}

		} catch (javax.naming.NamingException ne) {
			System.out
			.println("<JNDI>: failed to get Initial context from JNDI (naming exception)");
			ne.printStackTrace();
			return false;
		}
		return true;
	}

	/**
	 * @return
	 * @throws javax.naming.NamingException
	 */
	public static Context getInitialContext(String globalJNDIHostName)
			throws javax.naming.NamingException {
		
		//Create the JNDI context here.
		/*java.naming.factory.initial=org.jnp.interfaces.NamingContextFactory
		java.naming.factory.url.pkgs=org.jboss.naming:org.jnp.interfaces
		
		java.naming.provider.url=risk.ece.cmu.edu
		*/
		
		System.out.println("<JNDI>: The global JNDI server name is " + globalJNDIHostName );
		Properties props = new Properties();
		props.setProperty("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
		props.setProperty("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");
		props.setProperty("java.naming.provider.url", globalJNDIHostName);
		
				
		return new InitialContext(props);

	}

	/**
	 * @return value of NUM_SERVER_INSTANCES.
	 */
	public int getNUM_SERVER_INSTANCES() {
		return NUM_SERVER_INSTANCES;
	}
}