/**
 * 
 */
package edu.cmu.mse.vault.message;

import java.io.Serializable;

/**
 * Message object returned to the client
 * @version 1.0
 * @since 1.0
 * @author gzenarosa
 */
public class Message implements Serializable {
	private MessageKey key;
	private Serializable payload;
	
	/**
	 * Constructor
	 * @param key MessageKey to use for this message
	 * @param payload BLOB content of the message
	 * @since 1.0
	 */
	public Message(MessageKey key, Serializable payload) {
		this.key = key;
		this.payload = payload;
	}
	
	/**
	 * Constructor
	 * @param key MessageKey to use for this message
	 * @param payload BLOB content of the message
	 * @since 1.0
	 */
	public Message(MessageEntity me) {
		this.key = (MessageKey)me.getPrimaryKey();
		this.payload = me.getReplyMsg();
	}
	
	/**
	 * Retrieve the key for this message
	 * @return the MessageKey object
	 * @since 1.0
	 */
	public MessageKey getKey() {
		return key;
	}
	
	/**
	 * Retrieve the payload of this message
	 * @return
	 * @since 1.0
	 */
	public Serializable getPayload() {
		return payload;
	}

	/**
	 * Replace the payload of this message
	 * @return
	 * @since 1.0
	 */
	public void setPayload(Serializable payload) {
		this.payload = payload;
	}

}
