/*
 * $Id: VaultReplicationServer.java,v 1.10 2005/03/17 03:16:33 ssimanta Exp $
 *
 * Copyright Soumya
 */
package edu.cmu.mse.vault.repman;

import java.io.FileInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.Vector;

/**
 * @author Soumya
 *  
 */
public class VaultReplicationServer extends UnicastRemoteObject implements
        IVaultReplicationManager {

    //FIX need to get this from a properties file.
    public static final String VAULT_REP_MANAGER = "//localhost:10001/Vault_Rep_man";

    /* Holds all the properites for the repman */
    public static Properties properties;

    public static void main(String arg[]) {

        

        try {

            VaultReplicationServer replicationServer = new VaultReplicationServer();
            
            System.out.print("Starting the Vault Replication Manager with the following properites " + properties.toString() );            
            // "//localhost:10001/Vault_Rep_man";
            StringBuffer repmanager = new StringBuffer();

            repmanager.append("//")
            .append(properties.getProperty("repman.rmi.host"))
            .append(":")
            .append( properties.getProperty("repman.rmi.port") )
            .append("/")
            .append( properties.getProperty("repman.rmi.bindstring") );

            System.out.println("Starting to bind to " + repmanager.toString() );
            Naming.rebind( repmanager.toString(), replicationServer);
            

        } catch (RemoteException e) {

            e.printStackTrace();
        } catch (MalformedURLException e1) {

            e1.printStackTrace();
        }

    }


    /**
     * @throws RemoteException
     */
    protected VaultReplicationServer() throws RemoteException {
        super();
        loadProperties();
        System.out.println("A VaultReplicationServer created successfully at "
                + new Date());

    }

    /**
     * Load properties from the vaultjndi.properties file. Now, the properties
     * file contains three properties - MAX_NUM_SERVER_INSTANCES -
     * PERIOD_CHECK_JNDI - PREFIX_JNDI_NAME
     */
    public static void loadProperties() {
        // Read properties file.
        properties = new Properties();
        try {
            properties.load(new FileInputStream("repman.properties"));

        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see edu.cmu.mse.vault.repman.IVaultReplicationManager#getJNDIHostName()
     */
    public String getJNDIHostName() throws RemoteException {

        return properties.getProperty("global.jndi.host");

    }

    /*
     * (non-Javadoc)
     * 
     * @see edu.cmu.mse.vault.repman.IVaultReplicationManager#getAccountRefNames()
     */
    public List getAccountRefNames() throws RemoteException {
        return getRefNames("ebj.session.account");
    }

    /*
     * (non-Javadoc)
     * 
     * @see edu.cmu.mse.vault.repman.IVaultReplicationManager#getCustomerRefNames()
     */
    public List getCustomerRefNames() throws RemoteException {
        return getRefNames("ebj.session.customer");
    }

    /*
     * (non-Javadoc) returns a vector with the names of the
     * 
     * @see edu.cmu.mse.vault.repman.IVaultReplicationManager#getTransactionRefNames()
     */
    public List getTransactionRefNames() throws RemoteException {
        return getRefNames("ebj.session.transaction");
    }

    /*
     * refactored function for getTransactionRefNames, getAccountRefNames and
     * get
     */
    private List getRefNames(String typeOfSession) throws RemoteException {
        try {
            Vector theVector = new Vector();
            int replicaNum = getNumberOfReplicas();
            for (int i = 1; i <= replicaNum; i++)
                theVector.add(properties.getProperty("jndi.external.context.prefix")
                        + Integer.toString(i) + "/" + properties.getProperty(typeOfSession));

            return theVector;
        } catch (Exception e) {
            System.out.println(e);
            return null;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see edu.cmu.mse.vault.repman.IVaultReplicationManager#getNumberOfReplicas()
     */
    public int getNumberOfReplicas() throws RemoteException {
        return Integer.parseInt(properties.getProperty("replica.number"));
    }

    /*
     * (non-Javadoc)
     * 
     * @see edu.cmu.mse.vault.repman.IVaultReplicationManager#getCurrentActiveReplicas()
     */
    public int getCurrentActiveReplicas() throws RemoteException {
        // TODO Auto-generated method stub
        return Integer.parseInt(properties.getProperty("replica.number"));
    }
}